namespace MediaPlayer.Utils {
  public delegate bool FindDelegate (Gtk.Widget widget);

  public Gtk.Widget ? find_widget (Gtk.Widget widget, FindDelegate find_func) {
    if (find_func (widget)) {
      return widget;
    }
    if (widget is Gtk.Container) {
      foreach (var child in ((Gtk.Container) widget).get_children ()) {
        var found = find_widget (child, find_func);
        if (found != null)
          return found;
      }
    }
    return null;
  }

  public Gtk.Widget ? find_widget_by_name (Gtk.Widget widget, string name) {
    return find_widget (widget, (w) => w.name == name);
  }

  public Gtk.Widget ? find_widget_by_css_name (Gtk.Widget widget, string name) {
    return find_widget (widget, (w) => w.get_name () == name);
  }

  public int get_box_position (Gtk.Box box, Gtk.Widget child) {
    var i = 0;
    foreach (var _child in box.get_children ()) {
      if (_child == child)
        return i;
    }
    return -1;
  }

  public void replace_widget (Gtk.Widget old, Gtk.Widget replacement) {
    var container = old.parent;
    if (container is Gtk.Box) {
      var box = (Gtk.Box) container;
      var position = get_box_position (box, old);
      container.add (replacement);
      box.reorder_child (replacement, position);
      container.remove (old);
    } else {
      container.remove (old);
      container.add (replacement);
    }
  }

  public Gdk.Pixbuf round_pixbuff (Gdk.Pixbuf source, double roundness) {
    // That's a hack for GTK3. TODO: Remove when phosh will be moved to gtk4
    var width = source.width;
    var height = source.height;

    var surface = new Cairo.ImageSurface (Cairo.Format.ARGB32, width, height);
    var cr = new Cairo.Context (surface);
    var radius = int.max (width, height) * roundness;
    cr.new_path ();
    cr.arc (width - radius, radius, radius, deg (-90), deg (0));
    cr.arc (width - radius, height - radius, radius, deg (0), deg (90));
    cr.arc (radius, height - radius, radius, deg (90), deg (180));
    cr.arc (radius, radius, radius, deg (180), deg (270));

    cr.close_path ();
    cr.clip ();

    Gdk.cairo_set_source_pixbuf (cr, source, 0, 0);
    cr.paint ();
    return Gdk.pixbuf_get_from_surface (surface, 0, 0, width, height);
  }

  public Gdk.Pixbuf rect_pixbuff (Gdk.Pixbuf source) {
    // That's a hack for GTK3. TODO: Remove when phosh will be moved to gtk4
    var width = source.width;
    var height = source.height;
    if (width == height)
      return source;
    var max_dim = int.max (width, height);

    var surface = new Cairo.ImageSurface (Cairo.Format.ARGB32, max_dim, max_dim);
    var cr = new Cairo.Context (surface);
    Gdk.cairo_set_source_pixbuf (cr, source, 0, (width - height) / 2);
    cr.paint ();
    return Gdk.pixbuf_get_from_surface (surface, 0, 0, max_dim, max_dim);
  }

  public double deg (double deg) {
    return Math.PI / 180.0 * deg;
  }

  public struct SignalHelp {
    Object object;
    ulong signal;
  }

  public class PlayerReplacer : Gtk.Widget {
    public Player player = new Player ();
    private Gtk.Widget? old_player = null;

    construct {
      visible = false;
      var ts = new TimeoutSource (100);
      ts.set_callback (() => try_replace_player ());
      ts.attach ();
    }

    ~PlayerReplacer () {
      if (old_player != null)
        Utils.replace_widget (player, old_player);
      player.unbind ();
    }

    bool try_replace_player () {
      var anc = get_toplevel ();
      if (anc == null)
        return true;

      old_player = Utils.find_widget_by_name (anc, "PhoshDefaultMediaPlayer");
      if (old_player == null)
        return true;
      Utils.replace_widget (old_player, player);
      player.bind_to_player ((Gtk.Box) old_player);
      return false;
    }
  }
}
